<?php
/**
 * API de gestion des objets territoire.
 *
 * @package SPIP\TERRITOIRES\API
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

if (!defined('_TERRITOIRE_TIME_OUT_PEUPLEMENT')) {
	/**
	 * Time-out par défaut du traitement de peuplement asynchrone.
	 */
	define('_TERRITOIRE_TIME_OUT_PEUPLEMENT', 20);
}


/**
 * Renvoie la configuration du plugin, soit statique (non modifiable), soit utilisateur (modifiable).
 *
 * @api
 *
 * @param string $type_configuration Type de configuration. Prend les valeurs `statique` ou `utilisateur`.
 *
 * @return array Tableau de la configuration demandée.
 */
function territoire_configurer($type_configuration = 'statique') {

	if ($type_configuration === 'statique') {
		// Configuration de l'extraction des données REST
		$configuration = array(
			// liste des types de territoires (chaque type a une configuration)
			'types' => array('zone', 'country', 'subdivision', 'infrasubdivision'),
			// configuration de l'extraction des territoires de type zone
			'zone' => array(
				// -- liste des champs récupérés dans la réponse.
				'champs' => array(
					// L'index 'base' correspond aux colonnes dans la table spip_territoires sous la forme :
					// [index dans la réponse] = colonne de spip_territoires
					'base' => array(
						'code_num' => 'iso_territoire',
						'category' => 'categorie',
						'parent'   => 'iso_parent',
						'depth'    => 'profondeur_type',
						'label'    => 'iso_titre',
					),
					// L'index 'index' définit la racine des champs dans la réponse
					'index' => 'zones',
				),
				// -- liste des extras de type 'code' ou 'info' récupérés dans la réponse à un index différent de
				//    celui des champs et mis à jour dans la table spip_territoires_extras
				'extras' => array(
					// Pour le type 'code'
					'code' => array(
						// L'index 'champs' correspond aux colonnes dans la table spip_territoires_extras sous la forme :
						// [colonne de spip_territoires_extras] = index dans la réponse
						'champs'  => array(
							'extra'  => '#code_geoip', // La réponse ne contient pas ce champ : il est forcé à 'code_geopip'
							'valeur' => 'code',
						),
						// L'index 'index' définit la racine des champs extras dans la réponse
						'index'   => 'continents',
						// L'index 'cle_iso' identifie l'index dans la réponse qui représente le code iso, soit iso_territoire
						'cle_iso' => 'code_num',
					),
				),
			),
			'country' => array(
				'champs' => array(
					'base' => array(
						'code_alpha2'     => 'iso_territoire',
						'category'        => 'categorie',
						'code_continent'  => 'iso_continent',
						'label'           => 'iso_titre',
					),
					// -- liste des extras de type 'code' ou 'info' récupérés dans la réponse au même index que
					//    celui des champs et aussi mis à jour dans la table spip_territoires_extras.
					//    La forme des correspondance est identique à celle des champs de base.
					'extras' => array(
						'code' => array(
							'code_alpha3'     => 'code_iso3166_a3',
							'code_num'        => 'code_iso3166_num',
						),
						'info' => array(
							'capital'         => 'capital',
							'area'            => 'area',
							'population'      => 'population',
							'tld'             => 'tld',
							'code_4217_3'     => 'code_4217_3',
							'phone_id'        => 'phone_id'
						),
					),
					'index' => 'pays',
				),
			),
			'subdivision' => array(
				'champs' => array(
					'base' => array(
						'code_3166_2' => 'iso_territoire',
						'type'        => 'categorie',
						'country'     => 'iso_pays',
						'parent'      => 'iso_parent',
						'depth'       => 'profondeur_type',
						'label'       => 'iso_titre',
					),
					'index' => 'subdivisions',
				),
				'extras' => array(
					'code' => array(
						'champs'  => array(
							'extra'  => 'type_alter',
							'valeur' => 'code_alter',
						),
						'index'   => 'codes_alternatifs',
						'cle_iso' => 'code_iso',
					),
				),
			),
			'infrasubdivision' => array(
				'champs' => array(
					'base' => array(
						'code'        => 'iso_territoire',
						'type'        => 'categorie',
						'country'     => 'iso_pays',
						'parent'      => 'iso_parent',
						'depth'       => 'profondeur_type',
						'label'       => 'iso_titre',
						'parent_alt1' => 'iso_parent_alt',
					),
					'extras' => array(
						'info' => array(
							'lat' => 'latitude',
							'lon' => 'longitude',
						),
					),
					'index' => 'infrasubdivisions',
				),
				'extras' => array(
					'code' => array(
						'champs'  => array(
							'extra'  => 'type_alter',
							'valeur' => 'code_alter',
						),
						'index'   => 'codes_alternatifs',
						'cle_iso' => 'code_iso',
					),
				),
			),
		);
	} else {
		// Liste des objets associables à un territoire
		$configuration = array(
			'association_objets' => array(),
		);
	}

	return $configuration;
}

/**
 * Peuple soit les régions du monde, soit les pays ou soit les subdivisions d'un pays.
 * La fonction utilise les données fournies par Nomenclatures.
 *
 * @api
 *
 * @param string $type    Type de territoires. Prends les valeurs `zone`, `country`, `subdivision` ou `infrasubdivision`.
 * @param string $pays    Code ISO 3166-1 alpha2 du pays si le type est `subdivision` ou `infrasubdivision` sinon une chaine vide.
 * @param array  $options Tableau des options de peuplement:
 *                        - `force` : si `true` force le repeuplement même si le sha est identique (`false` par défaut).
 *                        - `extras`: tableau des types d'extras à peupler soit `code` pour les codes alternatifs
 *                                    et `info` pour le caractéristiques (table spip_territoires_extras)
 *
 * @return array Tableau retour de la fonction permettant de connaitre le résultat du traitement (utile pour l'affichage
 *               du message dans le formulaire de peuplement).
 */
function territoire_peupler($type, $pays, $options = array()) {

	// On initialise le retour à une erreur nok
	$retour = array(
		'ok'   => false,
		'arg'  => false,
		'sha'  => false,
		'type' => $type,
		'pays' => $pays,
	);

	// Détermination du mode de peuplement et insertion dans le retour pour les affichages
	$peuplement_asynchrone = isset($options['iteration']);
	$retour['sync'] = !$peuplement_asynchrone;

	// Le peuplement dépend du type :
	// - type = zone ou country : on charge l'ensemble des régions du monde ou l'ensemble des pays
	// - type = subdivision ou infrasubdivision : il faut préciser le pays (code ISO alpha2) pour lequel on charge toutes les subdivisions
	include_spip('inc/territoires_services');
	include_spip('inc/territoires_utils');
	if (type_pays_est_valide($type, $pays)) {
		// Lecture de la configuration statique du type de territoire
		include_spip('inc/config');
		$configuration = lire_config("territoires/${type}");

		// Identification de la meta et de la variable de consigne
		$meta = 'territoires_peuplement';
		$consigne = consigne_identifier($type, $pays);

		// Initialisation du numéro d'itération (toujours à 0 en mode synchrone).
		$iteration = empty($options['iteration']) ? 0 : $options['iteration'];
		$premiere_iteration = ($iteration === 0);
		// Calcul du timeout (utile uniquement en mode asynchrone)
		$timeout = time() + _TERRITOIRE_TIME_OUT_PEUPLEMENT;

		// Initialisation à vide des variables à transmettre d'une itération à une autre.
		$ids = array();
		$extras = array();
		$meta_extras = array();
		$sha_type = '';
		$territoires = array();
		$sauvegardes = array();
		$timestamp = array();

		// Phase 1 :
		// - Acquisition des données et vérification de l'utilité de les recharger ou pas.
		// - Si rechargement et première itération, on lance des pré-traitements de sauvegardes des modifications
		//   manuelles de territoires et de vidage des territoires à recharger.
		// - Si on est dans une nième itération, les pré-traitements laissent la place à la lecture des données de
		//   contexte.
		$type_identique = false;
		if ($premiere_iteration) {
			// Pour la première itération, mode asynchrone ou pas, on fait appel à l'API REST.
			$timestamp['debut'] = microtime(true);

			// Acquisition de la collection complete du couple (type, pays) via l'API REST
			// -- on précise si le mode est asynchrone afin de mettre en cache la collection dans ce cas.
			$collection = territoires_acquerir($type, $pays, array('cacher' => $peuplement_asynchrone));

			// On extrait la liste complète des territoires
			if (!empty($collection[$configuration['champs']['index']])) {
				$territoires = $collection[$configuration['champs']['index']];
			}
			$timestamp['acquisition'] = microtime(true);

			// Si le type de territoire est déjà chargé il possède un sha. Si le sha des data récupérées de Nomenclatures
			// possèdent le même sha alors on ne fait aucun traitement et on indique que la configuration n'a pas
			// changée. Dans ce cas, aucun traitement n'a lieu.
			if ($territoires) {
				$sha_type = sha1(json_encode($territoires));
				if (
					sha_est_identique($sha_type, $meta, $consigne)
					and empty($options['force'])
				) {
					// Le traitement va s'arrêter car il est inutile de recharger les mêmes données : on ne fait que consigner
					// le cas "d'erreur".
					$type_identique = true;
				} else {
					// On sait que le traitement va continuer on peut continuer les étapes de préalables à l'insertion.
					// -- on préserve les éditions manuelles et les liens pour les réinjecter ensuite lors du
					//    rechargement
					$sauvegardes = territoires_preserver($type, $pays, array('cacher' => $peuplement_asynchrone));
					$timestamp['preservation'] = microtime(true);

					// -- on vide les territoires avant de les remettre (inutile de gérer les erreurs
					//    car l'insertion les détectera).
					//    On gère aussi les infra subdivisions qui doivent être vidées au préalable si on vide
					//    les subdivisions parents.
					if (
						($type === 'subdivision')
						and territoire_est_peuple('infrasubdivision', $pays)
					) {
						territoire_depeupler('infrasubdivision', $pays);
					}
					territoire_depeupler($type, $pays);
					$timestamp['vidage'] = microtime(true);
				}
			}
		} else {
			// Nième itération : on est en mode asynchrone, on continue à partir des données stockées
			// dans des caches sécurisés.
			// -- lecture du cache de la collection complète correspondant au couple (type, pays).
			include_spip('inc/ezcache_cache');
			$cache = array(
				'type'     => $type,
				'pays'     => $pays,
				'fonction' => 'collection'
			);
			$collection = cache_lire('territoires', 'asynchrone', $cache);

			// -- lecture du cache des sauvegardes.
			include_spip('inc/ezcache_cache');
			$cache['fonction'] = 'sauvegarde';
			$sauvegardes = cache_lire('territoires', 'asynchrone', $cache);

			// -- lecture du contexte de chargement et extraction des territoires restant à insérer
			//    Le contexte contient toujours tous les index possibles.
			$cache['fonction'] = 'contexte';
			$contexte = cache_lire('territoires', 'asynchrone', $cache);
			if ($contexte) {
				$territoires = array_slice(
					$collection[$configuration['champs']['index']],
					$contexte['cle_fin'] + 1,
					null,
					true
				);
				$ids = $contexte['ids_crees'];
				$extras = $contexte['extras_crees'];
				$meta_extras = $contexte['meta_extras'];
				$sha_type = $contexte['sha_type'];
				$timestamp = $contexte['timestamp'];
			}
		}

		// Phase 2 :
		// - Insertion des territoires dans la base. Cette phase pouvant être longue elle peut nécessiter plusieurs
		//   itérations au sein d'une action asynchrone.
		$erreur = true;
		if (
			$territoires
			and !$type_identique
		) {
			$erreur_insertion = false;
			foreach ($territoires as $_cle => $_territoire) {
				// On initialise le territoire avec les noms de champs de la table spip_territoires
				$territoire = enregistrement_initialiser($_territoire, $type, $pays);

				// Si le code iso_continent est rempli c'est qu'on vient de charger les pays. Comme le code
				// du continent est le code alpha2 on le remplace par le code M49 pour être homogène.
				// -- la collection pays contient pour cela un bloc 'continents'.
				if (!empty($territoire['iso_continent'])) {
					$territoire['iso_continent'] = $collection['continents'][$territoire['iso_continent']]['code_num'];
				}

				// Intégrer les éventuelles modifications sauvegardées
				if (isset($sauvegardes['editions'][$territoire['iso_territoire']])) {
					// -- descriptif en fusionnant les traductions (priorité aux sauvegardes)
					// -- remise à 'oui' de l'indicateur d'édition
					$territoire['descriptif'] = traduction_fusionner(
						$sauvegardes['editions'][$territoire['iso_territoire']]['descriptif'],
						$territoire['descriptif']
					);
					$territoire['edite'] = 'oui';
				}

				if ($id = sql_insertq('spip_territoires', $territoire)) {
					// On consigne le couple (iso, id) pour rétablir les liens si besoin
					$ids[$territoire['iso_territoire']] = $id;

					// Si demandé et que la configuration le prévoit préparer le tableau des extras inclus dans
					// le bloc principal des territoires
					if (!empty($options['extras'])) {
						$source = $_territoire;
						$source['iso_territoire'] = $territoire['iso_territoire'];
						$extras = array_merge(
							$extras,
							extra_compiler(
								'interne',
								$options['extras'],
								$source,
								$configuration,
								$type,
								$pays,
								$meta_extras
							)
						);
					}
				} else {
					$erreur_insertion = true;
					break;
				}

				// Si le territoire a été inséré, on vérifie si on a atteint le timeout. Dans ce cas, on sauvegarde
				// le contexte de chargement dans un cache et on sort de la fonction en précisant que le traitement
				// n'est pas terminé. Ainsi, l'action appelante pourra réappeler cette même fonction.
				// Cela n'est valable que pour un appel asynchrone.
				if (
					$peuplement_asynchrone
					and (time() >= $timeout)
				) {
					// Consignation du contexte dans un cache
					$contexte = array(
						'cle_fin'      => $_cle,
						'ids_crees'    => $ids,
						'extras_crees' => $extras,
						'meta_extras'  => $meta_extras,
						'sha_type'     => $sha_type,
						'timestamp'    => $timestamp,
					);
					include_spip('inc/ezcache_cache');
					$cache = array(
						'type'     => $type,
						'pays'     => $pays,
						'fonction' => 'contexte'
					);
					cache_ecrire('territoires', 'asynchrone', $cache, $contexte);

					// Consignation du nombre d'itérations dans le retour de la fonction
					$retour['iter'] = $iteration + 1;

					return $retour;
				}
			}
			$timestamp['insertion'] = microtime(true);

			if (!$erreur_insertion) {
				// Phase 3 :
				// - on complète la collection des extras
				// - on insère tous les extras collectés pour les territoires chargés
				// - et on rétablit les liens des territoires existant avant le chargement
				if (!empty($options['extras'])) {
					// Si la configuration le prévoit préparer le tableau des extras inclus dans un bloc annexe
					if (!empty($configuration['extras'])) {
						$extras = array_merge(
							$extras,
							extra_compiler(
								'externe',
								$options['extras'],
								$collection,
								$configuration,
								$type,
								$pays,
								$meta_extras
							)
						);
					}

					// Ajouter tous les extras compilés pour le type de territoire.
					if ($extras) {
						sql_insertq_multi('spip_territoires_extras', $extras);
					}
					$timestamp['extras'] = microtime(true);
				}

				// On rétablit les liens vers les territoires et les logos
				// -- les liens avec les autres objets
				$config_lien = array(
					'table'    => 'spip_territoires_liens',
					'id_table' => 'id_territoire'
				);
				liens_retablir('liens', $sauvegardes, $ids, $config_lien);

				// -- les liens avec les logos
				$config_lien = array(
					'table'    => 'spip_documents_liens',
					'id_table' => 'id_objet'
				);
				liens_retablir('logos', $sauvegardes, $ids, $config_lien);
				$timestamp['retablissement'] = microtime(true);

				// Permettre à d'autres plugins de compléter le peuplement.
				// -- par exemple le plugin Contours rétablit les liens vers les contours GIS (spip_liens_gis),
				//    les objets GIS étant conservés si ils existent déjà.
				$flux = array(
					'args' => array(
						'type'        => $type,
						'pays'        => $pays,
						'sauvegardes' => $sauvegardes,
						'ids_crees'   => $ids,
					),
					'data' => array()
				);
				pipeline('post_peupler_territoire', $flux);

				// On stocke les informations de chargement dans une meta.
				$contenu = array(
					'sha' => $sha_type,
					'nbr' => count($territoires),
					'maj' => date('Y-m-d H:i:s'),
					'lic' => isset($collection['credits']) ? $collection['credits'] : array(),
					'ext' => $meta_extras,
				);
				ecrire_config("${meta}/${consigne}", $contenu);
				$timestamp['fin'] = microtime(true);
				$erreur = false;
			}

			// Log du traitement pour debug
			$duree = $timestamp['fin'] - $timestamp['debut'];
			spip_log("Les territoires (Type '${type}' - Pays '${pays}') ont été chargées en ${duree} s", 'territoires' . _LOG_INFO_IMPORTANTE);
			if (
				defined('_LOG_FILTRE_GRAVITE')
				and (_LOG_FILTRE_GRAVITE >= _LOG_DEBUG)
			) {
				$timestamp_debut = $timestamp['debut'];
				foreach ($timestamp as $_periode => $_timestamp) {
					if ($_periode !== 'debut') {
						$timestamp_fin = $_timestamp;
						$duree = ($timestamp_fin - $timestamp_debut) * 1000;
						$timestamp_debut = $timestamp_fin;
						spip_log("Période ${_periode}: ${duree} ms", 'territoires' . _LOG_DEBUG);
					}
				}
			}
		}

		// Si on arrive à ce point de sortie, c'est que le traitement est complètement fini ou est en erreur.
		// En mode asynchrone, dans les deux cas, on supprime les caches.
		if ($peuplement_asynchrone) {
			$filtres = array(
				'type'     => $type,
				'pays'     => $pays,
			);
			$caches = cache_repertorier('territoires', 'asynchrone', $filtres);
			if ($caches) {
				cache_vider('territoires', 'asynchrone', $caches);
			}
		}

		// Si le territoire est en erreur, on stocke le cas d'erreur.
		if ($erreur) {
			if (
				$type_identique
				and $premiere_iteration
			) {
				$retour['sha'] = true;
				spip_log("Les territoires de (Type '${type}' - Pays '${pays}') sont inchangés", 'territoires' . _LOG_AVERTISSEMENT);
			} else {
				$retour['ok'] = false;
				if (!$territoires) {
					spip_log("Aucun territoire pour (Type '${type}' - Pays '${pays}') retourné par Nomenclatures", 'territoires' . _LOG_ERREUR);
				}
			}
		} else {
			$retour['ok'] = true;
		}
	} else {
		$retour['arg'] = true;
		spip_log("Le couple (Type '${type}' - Pays '${pays}') est invalide", 'territoires' . _LOG_ERREUR);
	}

	return $retour;
}

/**
 * Appelle la fonction de peuplement de territoires en asynchrone.
 *
 * @api
 *
 * @param string $type    Type de territoires. Prends les valeurs `zone`, `country`, `subdivision` ou `infrasubdivision`.
 * @param string $pays    Code ISO 3166-1 alpha2 du pays si le type est `subdivision` ou `infrasubdivision` sinon une chaine vide.
 * @param array  $options Tableau des options de peuplement:
 *                        - `force` : si `true` force le repeuplement même si le sha est identique (`false` par défaut).
 *                        - `extras`: tableau des types d'extras à peupler soit `code` pour les codes alternatifs
 *                        et `info` pour le caractéristiques (table spip_territoires_extras)
 *
 * @return array Tableau retour de la fonction permettant de connaitre le résultat du traitement (utile pour l'affichage
 *               du message dans le formulaire de peuplement).
 */
function territoire_peupler_asynchrone($type, $pays, $options = array()) {

	// On initialise le retour à une ok ce qui est le cas le plus fréquent car on ne fait que créer le job.
	$retour = array(
		'ok'   => true,
		'arg'  => false,
		'sha'  => false,
		'type' => $type,
		'pays' => $pays,
		'sync' => false
	);

	include_spip('inc/territoires_services');
	if (type_pays_est_valide($type, $pays)) {
		// Construction de la chaine des arguments passés à l'action.
		// -- type et pays sont toujours fournis même si le pays est la chaine vide.
		$arguments = "${type}:${pays}";
		// -- ajout des éventuelles options : on passe les options par leur identifiant
		if (!empty($options['force'])) {
			$arguments .= ':force';
		}
		if (!empty($options['extras'])) {
			$arguments .= ':' . implode(':', $options['extras']);
		}
		// -- ajout du numéro d'itération
		$arguments .= ':' . (empty($options['iteration']) ? 'iter0' : "iter{$options['iteration']}");

		// Génération de l'URL qui lancera le peuplement en asynchrone.
		include_spip('inc/actions');
		$url = generer_action_auteur('peupler_territoires', $arguments, '', true, 0, true);

		// Lancement du peuplement en asynchrone.
		include_spip('inc/queue');
		$retour['ok'] = queue_lancer_url_http_async($url);
	} else {
		$retour['arg'] = true;
		spip_log("Le couple (Type '${type}' - Pays '${pays}') est invalide", 'territoires' . _LOG_ERREUR);
	}

	return $retour;
}

/**
 * Supprime de la base soit les régions du monde, soit les pays ou soit les subdivisions d'un pays.
 *
 * @api
 *
 * @param string $type    Type de territoires. Prends les valeurs `zone`, `country`, `subdivision` ou `infrasubdivision`.
 * @param string $pays    Code ISO 3166-1 alpha2 du pays si le type est `subdivision` ou `infrasubdivision` sinon une chaine vide.
 * @param array  $options Tableau des options de dépeuplement:
 *                        - `force` : si `true` force le vidage même si la meta n'est pas présente (cas d'erreur
 *                                    sur timeout par exemple). La valeur par défaut est `false`.
 *
 * @return array Tableau retour de la fonction permettant de connaitre le résultat du traitement (utile pour l'affichage
 *               du message dans le formulaire de peuplement).
 */
function territoire_depeupler($type, $pays = '', $options = array()) {

	// On initialise le retour à une erreur nok
	$retour = array(
		'ok'   => false,
		'arg'  => false,
		'sha'  => false,
		'type' => $type,
		'pays' => $pays,
		'sync' => true
	);

	// Le vidage dépend du type :
	// - type = zone ou country : on vide l'ensemble des régions du monde ou l'ensemble des pays
	// - type = subdivision ou infrasubdivision : il faut préciser le pays (code ISO alpha2) pour lequel on vide toutes les subdivisions
	include_spip('inc/territoires_services');
	include_spip('inc/territoires_utils');
	if (type_pays_est_valide($type, $pays)) {
		// Inutile de vider une table vide sauf si l'option de forçage est activée.
		if (
			territoire_est_peuple($type, $pays)
			or !empty($options['force'])
		) {
			// Identification de la variable de consigne
			$consigne = consigne_identifier($type, $pays);

			// Avant de vider la table on réserve la liste des id de territoire qui seront supprimés
			// de façon à vider ensuite les liens éventuels y compris ceux des logos.
			$from = 'spip_territoires';
			$where = array(
				'type=' . sql_quote($type),
			);
			if (type_est_subdivision($type)) {
				$where[] = 'iso_pays=' . sql_quote($pays);
			}
			if ($ids = sql_allfetsel('id_territoire', $from, $where)) {
				$ids = array_column($ids, 'id_territoire');

				$sql_ok = sql_delete($from, $where);
				if ($sql_ok !== false) {
					// Vider les extras si besoin
					$sql_ok = sql_delete('spip_territoires_extras', $where);

					if ($sql_ok !== false) {
						// Vider les liens éventuels avec les logos (on gère pas d'erreur)
						$where_logo = array(
							sql_in('id_objet', $ids),
							'objet=' . sql_quote('territoire')
						);
						sql_delete('spip_documents_liens', $where_logo);

						// Vider les liens éventuels avec les autres objets (on gère pas d'erreur)
						$where_lien = array(
							sql_in('id_territoire', $ids),
						);
						sql_delete('spip_territoires_liens', $where_lien);

						// Permettre à d'autres plugins de compléter le dépeuplement.
						// -- par exemple le plugin Contours supprime les liens vers les contours GIS (spip_liens_gis),
						//    mais pas les objets GIS.
						$flux = array(
							'args' => array(
								'type'           => $type,
								'pays'           => $pays,
								'ids_territoire' => $ids,
							),
							'data' => array()
						);
						pipeline('post_depeupler_territoire', $flux);

						// Supprimer la meta propre au pays même si le vidage des liens est en erreur.
						include_spip('inc/config');
						effacer_config("territoires_peuplement/${consigne}");
					}

					// Enregistrer le succès ou pas du déchargement de la table
					if ($sql_ok !== false) {
						// Succès complet
						$retour['ok'] = true;
						spip_log("Les territoires (Type '${type}' - Pays '${pays}') ont été vidés avec succès ainsi que les extras", 'territoires' . _LOG_DEBUG);
					} else {
						spip_log("Les territoires (Type '${type}' - Pays '${pays}') ont été vidés avec succès mais erreur lors du vidage des extras", 'territoires' . _LOG_ERREUR);
					}
				} else {
					spip_log("Erreur de vidage des territoires (Type '${type}' - Pays '${pays}')", 'territoires' . _LOG_ERREUR);
				}
			}
		} else {
			$retour['sha'] = true;
			spip_log("Aucun territoire (Type '${type}' - Pays '${pays}') à vider", 'territoires' . _LOG_AVERTISSEMENT);
		}
	} else {
		$retour['arg'] = true;
		spip_log("Le couple (Type '${type}' - Pays '${pays}') est invalide", 'territoires' . _LOG_ERREUR);
	}

	return $retour;
}

/**
 * Teste si un type de territoire est chargé en base.
 * La fonction lit la meta de chargement et non la table `spip_territoires`.
 *
 * @api
 *
 * @param string       $type Type de territoires. Prends les valeurs `zone`, `country`, `subdivision` ou `infrasubdivision`.
 * @param array|string $pays Code ISO 3166-1 alpha2 du pays si le type est `subdivision` ou `infrasubdivision` sinon une chaine vide.
 *
 * @return bool `true` si le territoire est chargé, `false` sinon.
 */
function territoire_est_peuple($type, $pays = '') {

	// Initialisation de la liste
	$est_peuple = false;

	// La liste des territoires chargés est en meta.
	include_spip('inc/config');
	$peuplement = lire_config('territoires_peuplement', array());
	if (
		include_spip('inc/territoires_utils')
		and type_est_subdivision($type)
	) {
		// Chaque pays chargé est un index du tableau
		if (isset($peuplement[$type])) {
			$est_peuple = array_key_exists($pays, $peuplement[$type]);
		}
	} else {
		$est_peuple = isset($peuplement[$type]);
	}

	return $est_peuple;
}

/**
 * Teste si les codes alternatifs ou les caractéristiques additionnelles d’un type de territoire sont chargées en base.
 * La fonction lit la meta de chargement et non la table `spip_territoires_extras`.
 *
 * @api
 *
 * @param string $type       Type de territoires. Prends les valeurs `zone`, `country`, `subdivision` ou `infrasubdivision`.
 * @param string $pays       Code ISO 3166-1 alpha2 du pays si le type est `subdivision` ou `infrasubdivision` sinon une chaine vide.
 * @param string $type_extra Type d'extra. Prends les valeurs `code` ou `info`.
 *
 * @return bool `true` si le territoire est chargé, `false` sinon.
 */
function territoire_extra_est_peuple($type, $pays, $type_extra) {

	// Initialisation de la liste
	$est_peuple = false;

	// La liste des territoires chargés est en meta.
	include_spip('inc/config');
	$peuplement = lire_config('territoires_peuplement', array());
	if (
		include_spip('inc/territoires_utils')
		and type_est_subdivision($type)
	) {
		// Chaque pays chargé est un index du tableau
		if (isset($peuplement[$type][$pays]['ext'])) {
			$est_peuple = in_array($type_extra, $peuplement[$type][$pays]['ext']);
		}
	} elseif (isset($peuplement[$type]['ext'])) {
		$est_peuple = in_array($type_extra, $peuplement[$type]['ext']);
	}

	return $est_peuple;
}
